@@ -1,19 +1,10 @@
 // Copyright (C) 2020 Mathieu Duponchelle <mathieu@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::glib;
 use gst::prelude::*;
@@ -23,7 +14,6 @@ use gst::{gst_debug, gst_error, gst_info, gst_log, gst_trace, gst_warning};
 use once_cell::sync::Lazy;
 
 use std::cmp;
-use std::convert::TryInto;
 use std::sync::{Mutex, MutexGuard};
 
 use serde::Deserialize;
@@ -194,7 +184,7 @@ impl State {
     ) {
         let buffer = buffer.get_mut().unwrap();
 
-        self.last_position = pts.zip(duration).map(|(pts, duration)| pts + duration);
+        self.last_position = pts.opt_add(duration);
 
         buffer.set_pts(pts);
 
@@ -221,8 +211,7 @@ impl JsonGstParse {
     ) -> Result<gst::FlowSuccess, gst::FlowError> {
         let mut state = self.state.lock().unwrap();
 
-        let drain;
-        if let Some(buffer) = buffer {
+        let drain = if let Some(buffer) = buffer {
             let buffer = buffer.into_mapped_buffer_readable().map_err(|_| {
                 gst::element_error!(
                     element,
@@ -234,10 +223,10 @@ impl JsonGstParse {
             })?;
 
             state.reader.push(buffer);
-            drain = false;
+            false
         } else {
-            drain = true;
-        }
+            true
+        };
 
         loop {
             let seeking = state.seeking;
@@ -257,13 +246,13 @@ impl JsonGstParse {
                     );
 
                     if !seeking {
-                        let data = data.to_string().clone();
+                        let data = data.to_string();
                         let mut events = state.create_events(element);
 
-                        let mut buffer = gst::Buffer::from_slice(data);
+                        let mut buffer = gst::Buffer::from_mut_slice(data.into_bytes());
 
                         if let Some(last_position) = state.last_position {
-                            if let Some(duration) = pts.map(|pts| pts.checked_sub(last_position)) {
+                            if let Ok(Some(duration)) = pts.opt_checked_sub(last_position) {
                                 events.push(
                                     gst::event::Gap::builder(last_position)
                                         .duration(duration)
@@ -274,12 +263,11 @@ impl JsonGstParse {
 
                         state.add_buffer_metadata(element, &mut buffer, pts, duration);
 
-                        let send_eos = state
-                            .segment
-                            .stop()
-                            .zip(buffer.pts())
-                            .zip(buffer.duration())
-                            .map_or(false, |((stop, pts), duration)| pts + duration >= stop);
+                        let send_eos = buffer
+                            .pts()
+                            .opt_add(buffer.duration())
+                            .opt_ge(state.segment.stop())
+                            .unwrap_or(false);
 
                         // Drop our state mutex while we push out buffers or events
                         drop(state);
@@ -346,11 +334,7 @@ impl JsonGstParse {
         pts: impl Into<Option<gst::ClockTime>>,
         mut state: MutexGuard<State>,
     ) -> MutexGuard<State> {
-        if pts
-            .into()
-            .zip(state.segment.start())
-            .map_or(false, |(pts, start)| pts >= start)
-        {
+        if pts.into().opt_ge(state.segment.start()).unwrap_or(false) {
             state.seeking = false;
             state.discont = true;
             state.replay_last_line = true;
@@ -503,7 +487,7 @@ impl JsonGstParse {
                     data: _data,
                 }) = serde_json::from_slice(line)
                 {
-                    last_pts = pts.zip(duration).map(|(pts, duration)| pts + duration);
+                    last_pts = pts.opt_add(duration);
                 }
             }
 
@@ -697,7 +681,11 @@ impl JsonGstParse {
         }
     }
 
-    fn perform_seek(&self, event: &gst::event::Seek, element: &super::JsonGstParse) -> bool {
+    fn perform_seek(
+        &self,
+        event: &gst::event::Seek<&gst::EventRef>,
+        element: &super::JsonGstParse,
+    ) -> bool {
         if self.state.lock().unwrap().pull.is_none() {
             gst_error!(CAT, obj: element, "seeking is only supported in pull mode");
             return false;
@@ -753,17 +741,11 @@ impl JsonGstParse {
         let pull = state.pull.as_ref().unwrap();
 
         if start_type == gst::SeekType::Set {
-            start = start
-                .zip(pull.duration)
-                .map(|(start, duration)| start.min(duration))
-                .or(start);
+            start = start.opt_min(pull.duration).or(start);
         }
 
         if stop_type == gst::SeekType::Set {
-            stop = stop
-                .zip(pull.duration)
-                .map(|(stop, duration)| stop.min(duration))
-                .or(stop);
+            stop = stop.opt_min(pull.duration).or(stop);
         }
 
         state.seeking = true;
@@ -949,6 +931,8 @@ impl ObjectImpl for JsonGstParse {
     }
 }
 
+impl GstObjectImpl for JsonGstParse {}
+
 impl ElementImpl for JsonGstParse {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
